/*
 * @author 天天敲代码(lmy)
 * @description 项目初始化
 * 1.加载预加载资源页面的资源
 * 2.设置背景颜色
 * 3.初始化数据(由于项目的数据太少，所有没有初始化)
 **/
import Phaser from '../libs/phaser';

export default class BootState extends Phaser.State {

    init() {
        console.log("启动 Boot");
    }

    preload() {
        var game = this.game;
        this.game.stage.backgroundColor = "#a8d4bf";

        /*  
          设置资源的base地址，如果不设置或者设置为空（ game.load.baseURL =""）,
          资源会加载项目assets目录下的资源文件
          若设置则会去服务器加载资源
         */
        //game.load.baseURL = "https://www.codeveryday.cn/static/source/";

        //加载preloadState页面需要的所有资源
        game.load.image('bgpreload', 'assets/images/bgpreload.png');
        game.load.image('logo', 'assets/images/logo.png');
        //加载动画图片
        game.load.image('loading', 'assets/images/loading.png');
        game.load.image('progress', 'assets/images/progress.png');
        game.load.image('snail', 'assets/images/snail.png');


        //用于封装资源加载异常信息
        this.errorList = []

        //onFileError资源加载异常的回调
        game.load.onFileError.add((key, file) => {

            //把异常信息添加到errorList数组中
            this.errorList.push(
                {
                    key: key,
                    errorMessage: file.errorMessage
                }
            )
        })


        //onFileComplete 资源加载完成（完成：包含失败和成功）回调
        game.load.onFileComplete.add((progress) => {

            //资源加载完成后，模拟 上报服务器资源加载情况
            if (progress == 100) {
                //模拟上报服务器
                if (this.errorList.length) {
                    console.log("失败的资源：" + JSON.stringify(this.errorList))
                }
            }
        })

    }

    create() {
        //跳转到资源预加载页面
        this.game.state.start("PreLoadState");
    }

}