/*
 * @author 天天敲代码(lmy)
 * @description 游戏页面，处理游戏的所有逻辑
 **/
import Phaser from '../libs/phaser';
import AllMusic from "../music/index";
import soundSeek from '../util/soundSeek';

//全局创建屏幕可用宽高，便于使用
const { windowWidth, windowHeight } = wx.getSystemInfoSync();

//标准高度，四分音符琴键对应的高度
const standardHeight = 150;

export default class GameState extends Phaser.State {

    init(props) {
        console.log("启动 GameState");
        // wx.clearStorage(); //清除所有微信存储的数据

        //开启arcade物理引擎        
        this.game.physics.startSystem(Phaser.Physics.ARCADE);

        //num表示第几关，已经通过，0表示游戏从第一关开始
        this.num = 0;
        this.num = wx.getStorageSync("num");
        if (!this.num) {
            this.num = 0;
        }
        //currentMusic为当前关卡，使用的歌曲数据
        //游戏类似消消乐一样，有无限多关。由于数据只做了8首歌，所以用8取模，就是数组AllMusic的下标，不断的循环
        this.currentMusic = AllMusic[this.num % 8];

        //用于生成琴键位置的x坐标，保证相邻的两个琴键x值不相同，也就是不在同一列，有四个值（0,1,2,3），
        this.currentColumnIndex = 0;

        //用于记录游戏的状态，0未开始，1.成功过关，-1.游戏失败
        this.gameStatus = 0;

    }

    create() {
        //创建背景
        this.createBackGround();

        //创建琴键和歌曲提示
        this.createKeyAndTips();

        //创建弹层，随着项目的迭代，此方法被废弃
        //this.createCover();

        //让scoreText位于显示对象的最上层，
        this.world.bringToTop(this.scoreText);

        //创建一个矩形精灵对象，用于与琴键碰撞检测，来判断游戏结束
        this.createpanel()
    }

    update() {

        //让琴键动起来，随之项目迭代，废弃
        // let keyArray = this.keyArray;
        // keyArray.forEach((item) => {
        //     item.y = item.y + 1;
        // })

        //碰撞检测，如果检测到碰撞，则游戏就结束，回调overlapCallback()去处理游戏失败逻辑
        this.game.physics.arcade.overlap(this.panelSprite, this.keyGroup, this.overlapCallback)
    }

    //游戏失败回调
    overlapCallback = () => {
        if (this.gameStatus != -1) {
            //游戏结束，禁止琴键点击，课程讲解中没有加这个代码，大家补上
            this.keyGroup.setAll("inputEnabled", false);
            //把所有琴键的速度置为0，禁止移动
            this.keyGroup.setAll("body.velocity", new Phaser.Point(0, 0));
            //console.log("game over");
            //游戏状态为-1，防止一直调用这个方法，一直从新生成显示对象，影响性能
            this.gameStatus = -1;
            //创建失败弹层
            this.createFailCover();
        }
    }
    createpanel() {

        let game = this.game;
        //创建一个矩形
        let panelGraphics = game.add.graphics(0, 0);
        panelGraphics.beginFill(0xffffff);
        panelGraphics.drawRect(0, 0, windowWidth, 20);
        panelGraphics.endFill();

        //使用panelGraphics生成一个精灵
        let panelSprite = this.game.add.sprite(0, windowHeight + 20, panelGraphics.generateTexture());
        panelGraphics.destroy();

        //panelSprite开启物理引擎
        game.physics.enable(panelSprite, Phaser.Physics.ARCADE);
        //把panelSprite挂载到this上，便于使用
        this.panelSprite = panelSprite;

    }
    //创建背景
    createBackGround() {
        let game = this.game;

        //创建背景图片，背景图片宽高设置成屏幕可用宽高
        let bgpreloadImg = game.add.image(0, 0, "bggame");
        bgpreloadImg.width = windowWidth;
        bgpreloadImg.height = windowHeight;

        //使用graphics画三条纵向的线条，把屏幕纵向等分成四份，每份就是琴键下落的轨道
        let lineGraphics = game.add.graphics(0, 0);
        lineGraphics.lineStyle(1, 0xffffff, 1);

        lineGraphics.moveTo(windowWidth / 4, 0);
        lineGraphics.lineTo(windowWidth / 4, windowHeight);

        lineGraphics.moveTo(windowWidth / 4 * 2, 0);
        lineGraphics.lineTo(windowWidth / 4 * 2, windowHeight);

        lineGraphics.moveTo(windowWidth / 4 * 3, 0);
        lineGraphics.lineTo(windowWidth / 4 * 3, windowHeight);

        //创建得分的文字，位于屏幕右上角位置，实时展示得分
        let scoreText = game.add.text(windowWidth - 100, 55, "得分:0", {
            font: '22px',
            fill: "#fa804d",
            align: "center"
        });

        //把scoreText挂载到this上，并在初始时先隐藏
        this.scoreText = scoreText;
        scoreText.exists = false;
    }
    //创建琴键和歌曲提示
    createKeyAndTips() {
        let game = this.game;

        //使用graphics画出一个矩形，并用其创建一个图片对象
        let beginGraphics = game.add.graphics(0, 0);
        beginGraphics.beginFill(0x8156e0);
        beginGraphics.drawRect(0, 0, windowWidth / 4, standardHeight);
        let beginImg = game.add.image(0, windowHeight - standardHeight * 2, beginGraphics.generateTexture());

        //为图片对象添加点击事件，点击后开始游戏，并隐藏或者销毁这个对象，最好的做法是销毁
        beginImg.inputEnabled = true;
        beginImg.events.onInputUp.add(() => {
            this.gameStatus = 1;
            this.songNameGroup.destroy();
            this.scoreText.exists = true;

            this.keyGroup.setAll("inputEnabled", true);

            //琴键的速度计算，根据曲子的速度来算，以每分钟60的速度为300作为基准
            this.keyGroup.setAll("body.velocity", new Phaser.Point(0, 300 * (this.currentMusic.speed / 60)));
            beginImg.exists = false;
        })

        //创建一个“开始”的文字，添加到beginImg中
        let beginText = game.add.text(windowWidth / 8, standardHeight / 2, "开始", {
            font: '40px',
            fill: "#fbc92e",
            align: "center"
        });
        beginText.anchor.set(0.5, 0.5);
        beginImg.addChild(beginText);

        //销毁beginGraphics
        beginGraphics.destroy();

        //把scoreText挂载到this上，
        this.beginImg = beginImg;

        //创建所有琴键，一次全部创建，这种方式简单直接，但是
        this.createKeys(false);

        let songNameGroup = game.add.group();
        this.songNameGroup = songNameGroup;
        songNameGroup.position.set(windowWidth / 2, 140);
        let songNameImg = game.add.image(0, 0, 'bgsongname', null, songNameGroup);
        songNameImg.scale.set(1.75, 1.5);
        songNameImg.anchor.set(0.5, 0.5);
        songNameImg.alpha = 0.9;
        let songNameText = game.add.text(0, 0, this.currentMusic.name, {
            font: '28px',
            fill: "#ffffff",
            align: "center"
        }, songNameGroup);
        songNameText.anchor.set(0.5, 0.5);
        let numText = game.add.text(0, -songNameImg.height / 2, "第" + (this.num + 1) + "关", {
            font: '22px',
            fill: "#c04d27",
            align: "center"
        }, songNameGroup);
        numText.anchor.set(0.5, 1);

    }

    /*  
     * 创建所有琴键，一次创建歌曲的所有琴键，这种方式简单直接，但是性能差，可以去优化
     * isNotFirst 是否是第一次调用这个方法，因为第一次调用，会有开始的按钮，对于琴键的y位置的计算有影响
    */
    createKeys(isNotFirst) {
        let game = this.game;

        //上一次点击琴键的信息，{}
        this.lastClick = null;

        //记录我的弹奏的每个音符数据，用于存储起来，包含琴键的名字和点击时间戳。{key: keyInfo.key, time: Date.now()}
        this.myMusic = null;

        //保存所有琴键对象，利于便利，还有用于琴键只可以顺序点击
        let keyArray = [];
        this.keyArray = keyArray;

        //把所有琴键都放进keyGroup这个组中，方便管理
        let keyGroup = game.add.group();
        this.keyGroup = keyGroup;
        //keyGroup开启物理引擎
        keyGroup.enableBody = true;
        keyGroup.physicBodyType = Phaser.Physics.ARCADE;

        //循环把歌曲所有的音符数据生成琴键
        let musics = this.currentMusic.music;
        for (let i = 0; i < musics.length; i++) {
            let keyInfo = musics[i];

            //计算琴键的高度，标准高度是四分音符的高度，四分音符是0.5怕，所有乘以2
            let keyHeight = standardHeight * keyInfo.length * 2;

            //画出琴键的矩形，宽度固定是屏幕的1/4，高度根据是计算出来的keyHeight
            let keyGraphics = game.add.graphics(0, 0);
            keyGraphics.beginFill(0x036ed6);
            keyGraphics.drawRect(0, 0, windowWidth / 4, keyHeight);


            //
            /* 
             *计算琴键的y坐标，第一个琴键的y值计算有两种情况
             *如果有开始按钮，根据开始按钮位置去计算，若没有，,则第一个琴键的位置为- keyHeight
             *也就是正好在屏幕外，因为琴键的锚点是左上角，也就是（0,0），
             *其他琴键的计算，根据上一个琴键计算，保证y值首尾相连，不叠加
            */
            let keyY = 0;
            if (keyArray.length == 0) {
                if (isNotFirst) {
                    keyY = - keyHeight;
                } else {
                    keyY = this.beginImg.y - keyHeight;
                }
            } else {
                keyY = keyArray[keyArray.length - 1].y - keyHeight;
            }
            //使用随机数去计算当前琴键的X位置，保证相邻的两个琴键x值不相同，x值只有四个值，所以模4
            let columnIndex = game.rnd.integerInRange(0, 400);
            columnIndex = columnIndex % 4;
            //若计算出当前的琴键x位置与上一个相同，则加1再去模4
            if (columnIndex == this.currentColumnIndex) {
                columnIndex = (columnIndex + 1) % 4;
            }
            this.currentColumnIndex = columnIndex;


            //使用上面计算出来的数据，创建琴键精灵，并直接加入到keyGroup组中
            let keySprite = game.add.sprite(windowWidth / 4 * columnIndex, keyY, keyGraphics.generateTexture(), null, keyGroup);

            //判断当前这个琴键是否是最后一个，最后一个琴键点击后，这一关过关
            if (i == musics.length - 1) {
                keySprite.isLast = true;
            }
            keySprite.keyInfo = keyInfo;

            //初始时候，禁止琴键可点击
            keySprite.inputEnabled = false;

            //根据琴键的名字，找到琴键的音频，空拍（0）是没有对应音频的
            let key = keyInfo.key.toLowerCase();
            if (key != "0") {
                let audio = wx.createInnerAudioContext();
                //取到当前音琴键属于那一组
                let groupNum = key.substring(key.length - 1);
                //由于变量不能是开头#、♭，为了防止程序有问题，特意把#转成h,把♭转成l
                let soundName = key.replace("#", "h").replace("♭", "l");
                audio.src = "assets/audio/" + groupNum + "/" + soundName + ".mp3";

                //把音频对象挂在到琴键上，方便使用
                keySprite.audio = audio;
            }

            //为琴键添加点击事件
            keySprite.events.onInputDown.add(() => {
                //判断当前元素是否是数组第一个，保证琴键的点击顺序，只可以顺序点击，点击之后从数据组删除
                if (keySprite == keyArray[0]) {
                    keyArray.shift();
                } else {
                    return;
                }
                //根据音符的时值和曲子的速率，计算音频播放的时间
                let timeOutLong = this.currentMusic.speed / 60 * keyInfo.length * 1000;

                //如果是第一个琴键则初始化，弹奏的数据为空数组
                if (keySprite.isFirst) {
                    this.myMusic = [];
                }
                //封装琴键弹奏的数组，包含琴键的名字和点击的时间戳
                let musicInfo = {
                    key: keyInfo.key,
                    time: Date.now()
                }

                //最后一个琴键的musicInfo数据中多一个timeLong，这是音符的时值
                if (keySprite.isLast) {
                    musicInfo.timeLong = timeOutLong;
                }
                //把数据添加到myMusic数组中
                this.myMusic.push(musicInfo);
                //console.log(JSON.stringify(musicInfo));

                /*   
                 * 超时时间，为音符的时值，
                 * 优化加上200的原因在于，
                 *这样可以有延长音，不至于听起来断断续续的
                 *最后一个音符不加200 
                 */
                let timeOutTime = timeOutLong + 200;
                if (keySprite.isLast) {
                    timeOutTime = timeOutLong
                }

                //添加一个超时定时器，用于销毁这个琴键和声音
                //如果是最后一个音符，则弹出这关成功的弹层，并存储过关的官关卡
                let timeOut = game.time.events.add(timeOutTime, () => {
                    keySprite.audio.destroy();
                    keySprite.destroy();
                    if (keySprite.isLast) {
                        this.createSuccessCover();
                        this.gameStatus = 0;
                        wx.setStorageSync("num", this.num + 1);
                    }
                });

                //播放琴键对应的音频，从校正后的开始时间，开始播放
                keySprite.audio.seek(soundSeek[key.toUpperCase()]);
                keySprite.audio.play();

                //琴键点击后有一个消失的动画
                let keyImgTween = game.add.tween(keySprite).to(
                    { alpha: 0 }, 400, "Linear", true
                );
                /* 
                    *计算弹奏每一个琴键的得分，最高分是5分，最低分是1分
                    *计算规则，点击当前琴键的时间，与点击上一个琴键的时间差，
                    *越接近上一个琴键的时值得分越高          
                */
                let score = 5;
                let clickTime = Date.now();
                let lengthTime = keyInfo.length * this.currentMusic.speed / 60 * 1000;

                //如果为false则为第一个琴键的点击，第一个琴键默认的得分为5分
                if (this.lastClick) {
                    let diffTime = clickTime - this.lastClick.clickTime;
                    let absTime = Math.abs(lengthTime - diffTime);
                    if (absTime <= 60) {
                        score = 5;
                    }
                    if (60 < absTime && absTime <= 90) {
                        score = 4;
                    }
                    if (90 < absTime && absTime <= 120) {
                        score = 3;
                    }
                    if (120 < absTime && absTime <= 150) {
                        score = 2;
                    }
                    if (150 < absTime) {
                        score = 1;
                    }
                }
                //封装琴键点击时间和时值，用于与下一个琴键的点击信息做比较，计算得分
                this.lastClick = {
                    clickTime: clickTime,
                    lengthTime: lengthTime
                }

                this.scoreText.text = "得分:" + (parseFloat(this.scoreText.text.replace("得分:", "")) + score);

                /*              
                 *点击后，把速度设置为0，让其不在下移动，
                 *课程讲解中使用的 keySprite.body.velocity.y=0
                 *由于pahser版本的不同，可能会没有作用，改成这样
                 */
                keySprite.body.velocity = new Phaser.Point(0, 0);;
            });

            //琴键上显示每个琴键的名字。
            let keyText = game.add.text(windowWidth / 8, keyHeight / 2, key, {
                font: '40px',
                fill: "#ffffff",
                align: "center"
            });
            keyText.anchor.set(0.5, 0.5);
            keySprite.addChild(keyText);
            keyGraphics.destroy();
            keyArray.push(keySprite);
        }


        //便利出来所有的空拍琴键，即key == "0"的琴键
        let whiteKeys = keyArray.filter((item) => {
            return item.keyInfo.key == "0";
        });
        //keyArray值保留非空拍的琴键，即key ！= "0"的琴键
        keyArray = keyArray.filter((item) => {
            return item.keyInfo.key != "0";
        });
        //找到第一个琴键，并为其挂在一个isFirst = true的属性
        keyArray[0].isFirst = true;

        //销毁所有的空拍琴键，调用group的remove()方法，第二个参数为true，表示会默认调用item的destroy()方法
        whiteKeys.forEach((item) => {
            keyGroup.remove(item, true);
        })
    }

    //创建弹层，随着项目的迭代，此方法被废弃
    createCover() {
        //this.createSuccessCover();
        // this.createFailCover();
    }

    //失败弹层
    createFailCover() {
        let game = this.game;

        //创建一个组，用于存放失败的弹层，这里为了规范，最好叫failGroup，大家明白就好了
        let sucessGroup = game.add.group();
        //位置屏幕居中
        sucessGroup.position.set(windowWidth / 2, windowHeight / 2);

        //弹层的北京图片
        let tancengImg = game.add.image(0, 0, 'tanceng', null, sucessGroup);
        tancengImg.scale.set(0.2, 0.2);
        tancengImg.anchor.set(0.5, 0.5);

        let headText = game.add.text(0, -85, "GAME OVER", {
            font: '38px',
            fill: "#e41a12",
            fontWeight: 'bold',
            align: "center"
        }, sucessGroup);
        headText.anchor.set(0.5, 0.5);
        let midText = game.add.text(0, -25, "您的" + this.scoreText.text, {
            font: '30px',
            fill: "#000000",
            fontWeight: 'bold',
            align: "center"
        }, sucessGroup);
        midText.anchor.set(0.5, 0.5);


        let backImg = game.add.image(-windowWidth / 2 + 10, -windowHeight / 2 + 10, "back", null, sucessGroup);
        backImg.scale.set(0.25, 0.25);

        backImg.inputEnabled = true;
        backImg.events.onInputUp.add(() => {
            this.state.start("NavState");
        });

        //重新开始的按钮，
        let nextImg = game.add.image(0, 66, "restart", null, sucessGroup);
        nextImg.anchor.set(0.5, 0.5);
        nextImg.scale.set(0.14, 0.14);

        //为重新开始按钮添加点击事件，分五步
        nextImg.inputEnabled = true;
        nextImg.events.onInputUp.add(() => {
            //1.销毁这个弹层对象
            sucessGroup.destroy();

            //2.游戏状态设置为1
            this.gameStatus = 1;

            //3.销毁keyGroup并重新创建所有的琴键
            this.keyGroup.destroy();
            this.createKeys(true);

            //4.重置得分
            this.scoreText.text = "得分:0";

            //5.让琴键可点击并向下移动
            this.keyGroup.setAll("inputEnabled", true);

            //琴键的速度计算，根据曲子的速度来算，以每分钟60的速度为300作为基准
            this.keyGroup.setAll("body.velocity", new Phaser.Point(0, 300 * (this.currentMusic.speed / 60)));
        })


        let nextText = game.add.text(0, 66, "重新开始", {
            font: '28px',
            fill: "#ffffff",
            align: "center"
        }, sucessGroup);
        nextText.anchor.set(0.5, 0.5);


    }


    //成功弹层
    createSuccessCover() {
        let game = this.game;

        //创建一个组，用于存放成功的弹层，
        let sucessGroup = game.add.group();
        //位置屏幕居中
        sucessGroup.position.set(windowWidth / 2, windowHeight / 2);

        let tancengImg = game.add.image(0, 0, 'tanceng', null, sucessGroup);
        tancengImg.scale.set(0.2, 0.2);
        tancengImg.anchor.set(0.5, 0.5);

        let headText = game.add.text(0, -85, "恭喜过关啦", {
            font: '38px',
            fill: "#e41a12",
            fontWeight: 'bold',
            align: "center"
        }, sucessGroup);
        headText.anchor.set(0.5, 0.5);
        let midText = game.add.text(0, -25, "您的" + this.scoreText.text, {
            font: '30px',
            fill: "#000000",
            fontWeight: 'bold',
            align: "center"
        }, sucessGroup);
        midText.anchor.set(0.5, 0.5);
        let backImg = game.add.image(-windowWidth / 2 + 10, -windowHeight / 2 + 10, "back", null, sucessGroup);
        backImg.scale.set(0.25, 0.25);

        backImg.inputEnabled = true;
        backImg.events.onInputUp.add(() => {
            this.state.start("NavState");
        });

        let nextImg = game.add.image(0, 66, "restart", null, sucessGroup);
        nextImg.anchor.set(0.5, 0.5);
        nextImg.scale.set(0.14, 0.14);

        //为下一关按钮添加点击事件，分五步
        nextImg.inputEnabled = true;
        nextImg.events.onInputUp.add(() => {
            //1.销毁这个弹层对象
            sucessGroup.destroy();

            //2.游戏状态设置为1
            this.gameStatus = 1;

            //3.销毁keyGroup并重新创建所有的琴键,要先修改关卡加1，并取到最新的曲子数据
            this.keyGroup.destroy();
            this.num = this.num + 1;
            this.currentMusic = AllMusic[this.num % 8];
            this.createKeys(true);

            //4.重置得分
            this.scoreText.text = "得分:0";

            //5.让琴键可点击并向下移动
            this.keyGroup.setAll("inputEnabled", true);

            //琴键的速度计算，根据曲子的速度来算，以每分钟60的速度为300作为基准
            this.keyGroup.setAll("body.velocity", new Phaser.Point(0, 300 * (this.currentMusic.speed / 60)));

        })

        let nextText = game.add.text(0, 66, "下一首", {
            font: '28px',
            fill: "#ffffff",
            align: "center"
        }, sucessGroup);
        nextText.anchor.set(0.5, 0.5);

        let saveImg = game.add.image(0, 140, "save", null, sucessGroup);
        saveImg.anchor.set(0.5, 0.5);
        saveImg.scale.set(0.14, 0.14);

        //为保存按钮添加事件
        saveImg.inputEnabled = true;
        saveImg.events.onInputUp.add(() => {

            //封装要保存的数据
            let historyItem = {
                name: this.currentMusic.name,
                time: Date.now(),
                num: this.num + 1,
                score: parseFloat(this.scoreText.text.replace("得分:", "")),
                myMusic: this.myMusic
            }


            //先取到已经保存的数据，是一个数组，并把新的数据添加到这个数组，重新保存
            let historyData = wx.getStorageSync("historyData");
            if (!historyData) {
                historyData = []
            }
            historyData.push(historyItem);
            wx.setStorageSync("historyData", historyData);
            //console.log(wx.getStorageSync("historyData"));

            /*     
            *销毁这个保存按钮，防止重复保存（同一数据保存多份）
            *但是严格来讲，在这里销毁不够严谨
            *放这里会导致bug，但是几率几乎为0，我们可以在这个方法的第一行代码执行 saveImg.inputEnabled = false
            *禁止点击事件，这样就可以绝对保证不会重复保存
            */
            saveImg.destroy();
            saveText.destroy();

            let tipText = game.add.text(windowWidth / 2, windowHeight - 70, " 保存成功 ", {
                font: '18px',
                fill: "#ffffff",
                align: "center",
                backgroundColor: "#999999"
            });
            tipText.anchor.set(0.5, 0.5);
            this.time.events.add(1000, () => {
                tipText.destroy();
            })

        })

        let saveText = game.add.text(0, 140, "保存", {
            font: '28px',
            fill: "#ffffff",
            align: "center"
        }, sucessGroup);
        saveText.anchor.set(0.5, 0.5);
        let nextSongText = game.add.text(0, 66 - nextImg.height / 2 + 4, AllMusic[(this.num + 1) % 8].name, {
            font: '16px',
            fill: "#999",
            align: "center"
        }, sucessGroup);
        nextSongText.anchor.set(0.5, 1);
    }

}