/*
 * @author 天天敲代码(lmy)
 * @description 资源加载页面，1.资源加载
 **/
import Phaser from '../libs/phaser';

export default class PreLoadState extends Phaser.State {

    constructor(keyState){
        super();
        this.keyState = keyState;
    }
    init(props) {
        console.log("启动 PreLoadState");
    }

    preload() {
        //加载资源
        this.loadAssets();

        //处理加载资源的动画
        this.createDisplayObj();


    }

    create() {
        //根据keyState的值判断将要跳转的页面，为了方便开发测试，上线时这个值为空或者未定义，跳转到NavState页面
        if(this.keyState){
          this.game.state.start(this.keyState);
        }else{
            this.game.state.start("NavState");
        }
        
    }

   

    loadAssets() {
        let game = this.game;

        //加载其他页面用到的所有资源
        game.load.image('bgnav', 'assets/images/bgnav.png');
        game.load.image('bgsongname', 'assets/images/bgsongname.png');
        game.load.image('bggame', 'assets/images/bggame.jpg');
        game.load.image('listhead', 'assets/images/listhead.png');
        game.load.image('backw', 'assets/images/backw.png');
        game.load.image('restart', 'assets/images/restart.png');
        game.load.image('save', 'assets/images/save.png');

        game.load.image('tanceng', 'assets/images/tanceng.png');
        game.load.image('listen', 'assets/images/listen.png');
        game.load.image('button', 'assets/images/button.png');
        game.load.image('buttonlan', 'assets/images/buttonlan.png');
        game.load.image('back', 'assets/images/back.png');
        game.load.image('btnhui', 'assets/images/btnhui.png');

        //处理加载失败的资源
        this.errorList = []
        game.load.onFileError.add((key, file) => {

            this.errorList.push(
                {
                    key: key,
                    errorMessage: file.errorMessage
                }
            )
        })


        game.load.onFileComplete.add((progress) => {

            //1.计算矩形宽度，2.重新画矩形遮罩，实现进度条的增加
            let length = progress/100*this.progressImg.width;
            this.maskGraphics.clear();
            this.maskGraphics.beginFill(0xffffff);
            this.maskGraphics.drawRect(-this.progressImg.width/2,-this.progressImg.height/2,length,this.progressImg.height);
            this.maskGraphics.endFill();

            if (progress == 100) {

                //模拟上报服务器
                if (this.errorList.length) {
                    console.log("失败的资源：" + JSON.stringify(this.errorList))
                }
            }
        })


    }

    createDisplayObj() {
        let {windowWidth, windowHeight} = wx.getSystemInfoSync()
        let game = this.game;
        //创建背景图片，背景图片宽高设置成屏幕可用宽高
        let bgpreloadImg = game.add.image(0, 0, "bgpreload");
        bgpreloadImg.width = windowWidth;
        bgpreloadImg.height = windowHeight;

        //加载动画，创建一个组，把所有元素放入这个组中
        let snailGroup = game.add.group();
        snailGroup.position.set(windowWidth / 2, windowHeight / 2);

        let progressImg = game.add.image(0, 0, 'progress', null, snailGroup);
        progressImg.anchor.set(0.5, 0.5);

        let progressScale = windowWidth * 0.7 / progressImg.width;
        progressImg.scale.set(progressScale, progressScale);
        
        //创建遮罩，遮罩是一个矩形
        let maskGraphics = game.add.graphics(0,0,snailGroup);
        maskGraphics.beginFill(0xffffff);
        maskGraphics.drawRect(-progressImg.width/2,-progressImg.height/2,progressImg.width,progressImg.height);
        maskGraphics.endFill();
        this.maskGraphics =maskGraphics ;

        //进度条的遮罩设置为maskGraphics，maskGraphics画出一个矩形
        progressImg.mask=maskGraphics;
        this.progressImg =progressImg;

        let snailImg = game.add.image(0, -progressImg.height / 2, "snail", null, snailGroup);
        snailImg.anchor.set(0.5, 1);
        snailImg.scale.set(progressScale, progressScale);


        let loadingImg = game.add.image(-15, progressImg.height / 2 + 10, "loading", null, snailGroup);
        loadingImg.anchor.set(0.5, 0);
        loadingImg.scale.set(progressScale, progressScale);

        //创建两个tween动画
        let snailImgTween = game.add.tween(snailImg).to({y: -progressImg.height / 2 - 15}, 800, "Linear", true, 0, -1, true);
        let loandingImgTween = game.add.tween(loadingImg).to({x:  15}, 800, "Linear", true, 0, -1, true);

        //开发者信息，包含一个图片和一段文字，把这两个放进一个组中
        let developInfoGroup = game.add.group();
        developInfoGroup.position.set(windowWidth/2,windowHeight-100);
        let logoImg = game.add.image(0,0,"logo",null,developInfoGroup);
        logoImg.scale.set(0.08,0.08)
        let infoText = game.add.text(45,12,"钢琴方块，由天天敲代码提供技术支持",{
            font:'16px',
            fill:"#ffffff",
            align:"center"
        },developInfoGroup)

       /*  把包含图片和文字的这个组，生成一个图片资源，并用这个资源创建一个图片对象，
        这样更利于去居中定位 */
        //（居中定位）扩展：可以通过developInfoGroup实际宽度和高度，然后设置developInfoGroup的坐标
        let delelopTexture = developInfoGroup.generateTexture();

        let developInfoGroupImg = game.add.image(windowWidth/2,windowHeight-2,delelopTexture);
        developInfoGroupImg.anchor.set(0.5,1)
        developInfoGroup.destroy()

    }
}